/*
 *	Copyright © Jan Bobrowski
 *
 *	This library is free software; you can redistribute and modify
 *	it under the terms of the GNU Lesser General Public License
 *	version 2.1 as published by the Free Software Foundation.
 */

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#if 0
#include <poll.h>
#else
#include <sys/select.h>
#endif
#include <sys/signal.h>
#include <assert.h>
#include <string.h>
#include <stdio.h>
#include "kbd.h"

#pragma GCC diagnostic ignored "-Wunused-result"

#define elemof(T) (sizeof T/sizeof*T)
#define endof(T) (T + elemof(T))

typedef unsigned char u8;

int getkey();
void term_raw();

char *nametbl[] = {
	"Enter", "BS", "Tab", "Esc", "Left", "Right", "Up", "Down", "Ins", "Del",
	"Home", "End", "PgUp", "PgDn", 0,0, "Begin", "WinL", "WinR", "Menu",
	0,0,0,0, 0,0,0,0, 0,0,0,0,
	"F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", "F11", "F12",
	"F13", "F14", "F15", "F16", "F17", "F18", "F19", "F20", "F21", "F22"
};

char *modif(int c)
{
	static char m[4];
	memset(m, '-', sizeof m - 1);
	if(c & kSHIFT) m[0] = 's';
	if(c & kALT) m[1] = 'a';
	if(c & kCTRL) m[2] = 'c';
	return m;
}

char *butt(int b)
{
	static char s[4];
	char *p = s;
	int m = 1;
	do {
		*p = '-';
		if(b & m)
			*p = 'o';
		m<<=1;
	} while(++p < s + sizeof s - 1);
	return s;
}

char *key_name(int k)
{
	static char b[48];
	char *p = b;
	k &= ~kMODIF;
	if(k >= 0x1000000 && k - 0x1000000 < elemof(nametbl))
		return nametbl[k-0x1000000];
	if(k == 0x20)
		return "Space";
	if(k == kMOUSE) {
		static char *a[8] = {"press", "release", "drag", "move", "wheel"};
		const struct mouse_event *e = getmouse();
		p += sprintf(p, "Mouse %-7s %s", a[e->code - 1], butt(e->button));
		p += sprintf(p, " %s %d:%d:%d", butt(e->state), e->x, e->y, e->wy);
		return b;
	}

	switch (k & 0xf000000) {
	case kTSIZE:
		sprintf(b, "Size %d:%d", k & 0xfff, k>>12 & 0xfff);
		return b;
	case kCPOS:
		sprintf(b, "Position %d:%d", k & 0xfff, k>>12 & 0xfff);
		return b;
	case kDECID:
		return "kDECID";
	case kTERM_T:
		return "kTERM_T";
	}

	if(k < 0x20 || k >= 127 && k < 0xA0)
		return "???";

	if(!getkey_utf8 || k < 0x80)
		*p++ = k;
	else {
		if(k < 0x800)
			*p++ = 0xC0 | k>>6;
		else {
			if(k < 0x10000)
				*p++ = 0xE0 | k>>12;
			else
				*p++ = 0xF0 | k>>18, *p++ = 0x80 | k>>12 & 0x3F;
			*p++ = 0x80 | k>>6 & 0x3F;
		}
		*p++ = 0x80 | k & 0x3F;
	}
	*p = 0;
	return b;
}

static void theend()
{
	write(1,"\e[?1000l", 8);
}

static void sigint(int n)
{
	exit(1);
}

int main(int argc, char **argv)
{
	getkey_utf8 = 1;
	if(argc > 1 && !strcmp(argv[1], "-u"))
		getkey_utf8 = 0; // disable utf-8

	term_raw();
	atexit(theend);
	signal(SIGINT, sigint);
	signal(SIGQUIT, SIG_IGN);
//	write(1,"\e[?1000h\e[?1003h", 16);
	write(1,"\e[?1000h\e[?1002h", 16);
//	write(1,"\e[?1000h", 8);
//	write(1,"\e[?9h", 5);

	for(;;) {
		int c = getkey();
		if(c == kERROR)
			break;
		if(c <= 0) {
#if 0
			struct pollfd pfd;
			pfd.fd = 0;
			pfd.events = POLLIN;
			poll(&pfd, 1, c == kMORE ? KBD_WAIT_TO_MS : -1);
#else
			fd_set rfds;
			struct timeval tv;
			tv.tv_sec = 0;
			tv.tv_usec = KBD_WAIT_TO_MS * 1000;
			FD_ZERO(&rfds);
			FD_SET(0, &rfds);
			select(1, &rfds, NULL, NULL, &tv);
#endif
			continue;
		}
		printf("%s %7X %-16s\n", modif(c), c&kMASK, key_name(c));

		if(c == (kCTRL | 'D'))
			break;
	}
}
