/*
 *	Copyright © Jan Bobrowski
 *
 *	This library is free software; you can redistribute and modify
 *	it under the terms of the GNU Lesser General Public License
 *	version 2.1 as published by the Free Software Foundation.
 */

#include <stdlib.h>
#include <fcntl.h>
#include <poll.h>
#include <signal.h>
#include <stdio.h>
#include <unistd.h>
#include <termios.h>

#pragma GCC diagnostic ignored "-Wunused-result"

void term_raw();

static int conf_mode, conf_mouse, conf_oneshot;

static void restore()
{
	if(conf_mode == 1) write(1, "\33>", 2);
	if(conf_mouse) write(1, "\33[?1000l", 8);
}

static void sigint(int n)
{
	exit(1);
}

int main(int argc, char **argv)
{
	struct pollfd pfd;

	for(;;) {
		char c = getopt(argc, argv, "anm1?");
		if(c == EOF) break;
		if(c == 'a') conf_mode = 1;
		if(c == 'n') conf_mode = 2;
		if(c == 'm') conf_mouse = 1;
		if(c == '1') conf_oneshot = 1;
		if(c == '-' || c == '?') {
			printf("-a\tset application mode\n-n\tset numeric keypad mode\n"
				"-m\tinit mouse\n-1\tonce\n");
			return 0;
		}
	}

	pfd.events = POLLIN;

	term_raw();

	if(conf_mode == 1)
		write(1, "\33=", 2);
	else if(conf_mode == 2)
		write(1, "\33>", 2);
	if(conf_mouse)
		write(1, "\33[?1000h", 8);
	write(1, "\r", 1);

	atexit(restore);
	signal(SIGINT, sigint);
	signal(SIGQUIT, SIG_IGN);
	pfd.fd = 0;
	for(;;) {
		unsigned char c;
		int v;
		poll(&pfd, 1, -1);
		for(;;) {
			v = read(0, &c, 1);
			if(v > 0) {
				char b[8], *t = b;
				switch(c) {
					case ' ': t = "SP"; break;
					case 27: t = "ESC"; break;
					case 8: t = "BS"; break;
					case 9: t = "TAB"; break;
					case 10: t = "LF"; break;
					case 13: t = "CR"; break;
					case 127: t = "DEL"; break;
					default:
						if(c > ' ' && c < 127)
							b[0] = c, b[1] = 0;
						else
							sprintf(b, "%02X ", c);
				}
				printf("%s ", t); fflush(stdout);
				continue;
			} else if(!v)
				return 0;
			if(0 == poll(&pfd, 1, 100))
				break;
		}
		printf("\n");
		if(conf_oneshot)
			break;
	}
	return 0;
}
